'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _bluebird = require('bluebird');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
// This is the entry script for the CLI utility.
//

//region Imports
var _console = console;
var log = _console.log;

var c = require('chalk');
//endregion

process.on('unhandledRejection', function (e) {
  throw e;
});

function cli() {
  var _this = this;

  main().then((0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee() {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this);
  })));
  return true;
}

module.exports = cli;

////////////////////////////////////////////////////////////////////////////////

var main = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee2() {
    var devDeps, runtimeDeps, convertToPackageJsonDepFormat;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            convertToPackageJsonDepFormat = function convertToPackageJsonDepFormat(deps) {
              var o = {};
              deps.forEach(function (_ref) {
                var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

                var name = _ref2[0];
                var version = _ref2[1];
                return o[name] = version || '*';
              });
              return o;
            };

            welcome();

            _context2.next = 4;
            return confirmLocation();

          case 4:

            initPackageJson();

            _context2.next = 7;
            return addGulpScriptToPackageJson();

          case 7:
            _context2.next = 9;
            return createGulpfile();

          case 9:

            // TODO(vjpr): Set to current versions from npm.
            devDeps = [['babelator', '0.2.8'], ['gulp', 'gulpjs/gulp.git#4.0'], ['babel-preset-live', '0.0.4'], ['babel-preset-hmre', '1.0.1']];
            runtimeDeps = [['babel-register', '^6'], // For `mocha:src` tests and if we add a `modules` dir.
            ['babel-runtime', '^5'], ['bluebird', '^3'], ['babel-polyfill', '^6']];
            _context2.next = 13;
            return addNpmDependenciesToPackageJson({
              devDependencies: convertToPackageJsonDepFormat(devDeps),
              dependencies: convertToPackageJsonDepFormat(runtimeDeps),
              reject: {
                devDependencies: ['babel-preset-react', 'babel-preset-es2015', 'babel-preset-stage-0'],
                dependencies: ['babel-preset-react', 'babel-preset-es2015', 'babel-preset-stage-0']
              }
            });

          case 13:
            _context2.next = 15;
            return installNpmDependencies({ devDeps: devDeps, runtimeDeps: runtimeDeps });

          case 15:

            gulpBootstrap();

            printHelp();

          case 17:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));
  return function main() {
    return ref.apply(this, arguments);
  };
}();

////////////////////////////////////////////////////////////////////////////////
// Steps
////////////////////////////////////////////////////////////////////////////////

// TODO
// - Use init-package-json.
// - Use promises.

function welcome() {

  log(c.bold('Welcome to the Babelator CLI'));
  log(c.grey('- Babelator will first create a minimal npm module with gulp installed, and Babelator\'s gulp tasks available.'));
  log(c.grey('- The rest of Babelator\'s bootstrapping occurs through gulp tasks by running `gulp bootstrap`.'));
  log(c.grey('- What about existing files? Babelator will confirm all changes to existing files making it super safe and easy to babelate existing projects.'));
}

var confirmLocation = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee3() {
    var inquirer, cwd, ans;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:

            logHeading('1. Confirm module location');

            inquirer = require('inquirer');
            cwd = require('cwd');
            _context3.next = 5;
            return prompt([{ type: 'confirm', name: 'confirm', message: 'Confirm your module\'s location: \'' + cwd() + '/package.json\'' }]);

          case 5:
            ans = _context3.sent;

            if (!ans.confirm) {
              log('Exiting');
              process.exit(1);
            }

          case 7:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));
  return function confirmLocation() {
    return ref.apply(this, arguments);
  };
}();

function initPackageJson() {

  logHeading('2. Create package.json if it does not exist');

  var fs = require('fs');
  var cwd = require('cwd');

  var _require = require('child_process');

  var spawnSync = _require.spawnSync;

  var packageJsonExists = fs.existsSync(cwd('package.json'));
  if (!packageJsonExists) {
    console.log('File `package.json` does not exist. A basic file will be created now. Don\'t worry, the contents will be iteratively modified further on.');
    log('=> npm init -y');
    // NOTE: -y - Use defaults and do not prompt user.
    spawnSync('npm', ['init', '-y'], { stdio: 'inherit' });
  } else {
    log('File `package.json` already exists.');
  }
}

var addGulpScriptToPackageJson = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee4() {
    var _require2, promisify, modifyPackageJson, _;

    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:

            logHeading('3. Add gulp command to package.json scripts hash');

            _require2 = require('bluebird');
            promisify = _require2.promisify;
            modifyPackageJson = require('./util/modifyPackageJson');
            _ = require('lodash');
            _context4.next = 7;
            return promisify(modifyPackageJson)(function (json) {
              _.set(json, 'scripts.gulp', './node_modules/gulp/bin/gulp.js');
              return json;
            });

          case 7:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));
  return function addGulpScriptToPackageJson() {
    return ref.apply(this, arguments);
  };
}();

var createGulpfile = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee5() {
    var _require3, promisify, replaceFile, cwd, gulpfileContents;

    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:

            logHeading('4. Add Babelator tasks to gulpfile.js');

            _require3 = require('bluebird');
            promisify = _require3.promisify;
            replaceFile = require('./util/replaceFile');
            cwd = require('cwd');
            gulpfileContents = "var gulp = require('gulp')\n" + "require('babelator')(gulp)\n";
            _context5.next = 8;
            return promisify(replaceFile)(cwd('gulpfile.js'), gulpfileContents);

          case 8:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));
  return function createGulpfile() {
    return ref.apply(this, arguments);
  };
}();

var addNpmDependenciesToPackageJson = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee6(_ref3) {
    var dependencies = _ref3.dependencies;
    var devDependencies = _ref3.devDependencies;
    var reject = _ref3.reject;

    var _require4, promisify, modifyPackageJson, _;

    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:

            logHeading('5. Add npm dependencies to package.json');

            _require4 = require('bluebird');
            promisify = _require4.promisify;
            modifyPackageJson = require('./util/modifyPackageJson');
            _ = require('lodash');
            _context6.next = 7;
            return promisify(modifyPackageJson)(function (json) {
              json.dependencies = _.merge({}, json.dependencies, dependencies);
              json.devDependencies = _.merge({}, json.devDependencies, devDependencies);
              reject.dependencies.forEach(function (d) {
                delete json.dependencies[d];
              });
              reject.devDependencies.forEach(function (d) {
                delete json.devDependencies[d];
              });
              return json;
            });

          case 7:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));
  return function addNpmDependenciesToPackageJson(_x) {
    return ref.apply(this, arguments);
  };
}();

var installNpmDependencies = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee7(_ref4) {
    var devDeps = _ref4.devDeps;
    var runtimeDeps = _ref4.runtimeDeps;

    var _require5, join, _require6, execSync, _require7, spawnSync, fs, s, _, cwd, allDeps, npmGlobalRoot, choices, _ans, _ans2, ans, depsToLink, depsToInstall, depsToLinkNames, depsToInstallNames, _ans3, cmd;

    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:

            logHeading('6. Install npm dependencies');

            log(c.grey('- `npm install` takes some time. To speed up the process you can use `npm link` instead.'));
            log(c.grey('- For this to work you need to checkout the repo and `npm link` it.'));
            log(c.grey('  - E.g. `git checkout https://github.com/vjpr/babelator && npm install && npm link`'));
            log(c.grey('- For monorepos like babel you need to `npm link` the individual packages.'));
            log(c.grey('  - E.g. `git checkout https://github.com/babel/babel && cd packages/babel-preset-es2015 && npm install && npm link`'));
            log(c.grey('Tip: Use `npm_lazy` module to locally cache npm modules to speed up installs.'));
            log();

            _require5 = require('path');
            join = _require5.join;
            _require6 = require('child_process');
            execSync = _require6.execSync;
            _require7 = require('child_process');
            spawnSync = _require7.spawnSync;
            fs = require('fs');
            s = require('string');
            _ = require('lodash');
            cwd = require('cwd');

            // Create list of dev deps and runtime deps in one list for further processing.

            devDeps = devDeps.map(function (_ref5) {
              var _ref6 = (0, _slicedToArray3.default)(_ref5, 2);

              var name = _ref6[0];
              var version = _ref6[1];
              return { name: name, version: version, dev: true };
            });
            runtimeDeps = runtimeDeps.map(function (_ref7) {
              var _ref8 = (0, _slicedToArray3.default)(_ref7, 2);

              var name = _ref8[0];
              var version = _ref8[1];
              return { name: name, version: version, dev: false };
            });
            allDeps = devDeps.concat(runtimeDeps);

            // Check if modules are npm linked.

            npmGlobalRoot = s(execSync('npm root -g', { encoding: 'utf8' })).trim().s;
            //const linkedDeps  = allDeps.map(({name, version, dev}) => fs.existsSync(join(npmGlobalRoot, name)) ? true : false)

            choices = allDeps.map(function (_ref9) {
              var name = _ref9.name;
              var version = _ref9.version;
              var dev = _ref9.dev;

              var linkedPath = join(npmGlobalRoot, name);
              var isLinked = fs.existsSync(linkedPath);
              var realPath = isLinked ? fs.realpathSync(join(npmGlobalRoot, name)) : null;
              var localModulePath = join(cwd(), 'node_modules', name);
              var installed = fs.existsSync(localModulePath);
              var installedViaLink = installed ? fs.lstatSync(localModulePath).isSymbolicLink() : false;
              var installStatusStr = c.red(' not installed');
              if (installedViaLink) {
                installStatusStr = c.green(' already installed via link');
              } else if (installed) {
                installStatusStr = c.green(' already installed');
              }

              return {
                name: s(name + c.yellow('' + (dev ? ' devDep' : '')) + installStatusStr).padRight(50),
                value: { name: name, version: version, dev: dev },
                checked: !installed, // Used for filtering defaults.
                installed: installed,
                realPath: realPath,
                installedViaLink: installedViaLink,
                isLinked: isLinked
              };
            });

            if (!_.every(choices, function (ch) {
              return ch.installed;
            })) {
              _context7.next = 29;
              break;
            }

            _context7.next = 26;
            return prompt({ name: 'confirm', type: 'confirm', message: 'All dependencies are installed. Do you still want to install/link any?', default: false });

          case 26:
            _ans = _context7.sent;

            if (_ans.confirm) {
              _context7.next = 29;
              break;
            }

            return _context7.abrupt('return');

          case 29:
            _context7.next = 31;
            return prompt([{
              name: 'deps',
              type: 'checkbox',
              message: 'Which dependencies do you want to install or link (you can choose to install or link next)?',
              choices: choices
            }]);

          case 31:
            _ans2 = _context7.sent;

            // TODO(vjpr): Difference
            choices = _.intersectionWith(choices, _ans2.deps, function (a, b) {
              return a.value.name === b.name;
            });

            if (choices.length) {
              _context7.next = 37;
              break;
            }

            log();
            log('Nothing to install.');
            return _context7.abrupt('return');

          case 37:

            choices.forEach(function (ch) {
              ch.name = ch.name + (ch.isLinked ? c.grey(' Globally linked at: ' + ch.realPath) : c.yellow(' not globally linked'));
              ch.checked = ch.isLinked;
            });

            padlog('Global dependencies are installed here:', npmGlobalRoot);

            _context7.next = 41;
            return prompt([{
              name: 'depsToLink',
              type: 'checkbox',
              message: 'Which dependencies do you want to `npm link` instead of `npm install`?',
              choices: choices
            }]);

          case 41:
            ans = _context7.sent;
            depsToLink = ans.depsToLink;
            depsToInstall = _.differenceWith(choices, depsToLink, function (a, b) {
              return a.value.name === b.name;
            });
            depsToLinkNames = _.map(depsToLink, 'name');
            depsToInstallNames = _.map(depsToInstall, function (d) {
              return d.value.name + (d.value.version ? '@' + d.value.version : '');
            });

            log();

            if (depsToLink.length) log('These dependencies will be linked:', c.yellow(depsToLinkNames.join(', ')));

            if (depsToInstall.length) log('These dependencies will be installed:', c.yellow(depsToInstallNames.join(', ')));

            log();

            _context7.next = 52;
            return prompt([{ type: 'confirm', name: 'confirm', message: 'Do you want to begin install/link phase?' }]);

          case 52:
            _ans3 = _context7.sent;

            if (_ans3.confirm) {
              cmd = undefined;

              if (depsToInstall.length) {
                cmd = 'npm install ' + depsToInstallNames.join(' ');
                padlog('=>', cmd);
                spawnSync('npm', ['install'].concat(depsToInstallNames), { stdio: 'inherit' });
              }
              if (depsToLink.length) {
                cmd = 'npm link ' + depsToLinkNames.join(' ');
                padlog('=>', cmd);
                spawnSync('npm', ['link'].concat(depsToLinkNames), { stdio: 'inherit' });
              }
            }

          case 54:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));
  return function installNpmDependencies(_x2) {
    return ref.apply(this, arguments);
  };
}();

function gulpBootstrap() {
  var _require8 = require('child_process');

  var spawnSync = _require8.spawnSync;

  var cmd = 'npm run gulp bootstrap';

  log();
  log('=>', cmd);

  spawnSync('npm', ['run', 'gulp', 'bootstrap'], { stdio: 'inherit' });
}

function printHelp() {

  // TODO
  padlog('Bootstrapping complete!');

  log(c.bold.yellow('Getting started'));
  log();
  log('  npm run gulp test-watch ' + c.grey('Compiles and run tests on file change. Useful for development!'));
  log('  npm run gulp nodemon ' + c.grey('Run package.json main script with nodemon'));
  log('  npm run gulp release ' + c.grey('Interactive CLI to release new version on npm'));
  log('  npm-check -us ' + c.grey('Interactively update all your dependencies to their latest version'));
  log();
}

////////////////////////////////////////////////////////////////////////////////
// Utilities
////////////////////////////////////////////////////////////////////////////////

function logHeading(str) {
  log();
  log(c.underline(str));
  log();
}

function prompt(qs) {
  var inquirer = require('inquirer');
  return new _promise2.default(function (resolve, reject) {
    inquirer.prompt(qs, function (answers) {
      resolve(answers);
    });
  });
}

function padlog() {
  log();
  log.apply(undefined, arguments);
  log();
}